@@ -1,24 +1,16 @@
-import logging
-import multiprocessing
-import os
-import tempfile
-from concurrent import futures
-from concurrent.futures import ProcessPoolExecutor
-from functools import partial
-from pathlib import Path
-from shutil import rmtree
-from typing import Any, Union, overload
+from typing import Any, Literal, overload
 
-from typing_extensions import Literal
-
-from cluster_tools._utils.warning import enrich_future_with_uncaught_warning
+from cluster_tools.executor_protocol import Executor
+from cluster_tools.executors.dask import DaskExecutor
 from cluster_tools.executors.debug_sequential import DebugSequentialExecutor
-from cluster_tools.executors.multiprocessing import MultiprocessingExecutor
-from cluster_tools.executors.pickle import PickleExecutor
+from cluster_tools.executors.multiprocessing_ import MultiprocessingExecutor
+from cluster_tools.executors.pickle_ import PickleExecutor
 from cluster_tools.executors.sequential import SequentialExecutor
 from cluster_tools.schedulers.cluster_executor import (
-    ClusterExecutor,
-    RemoteOutOfMemoryException,
+    ClusterExecutor,  # noqa:  F401 `cluster_tools.schedulers.cluster_executor.ClusterExecutor` imported but unused;
+    RemoteOutOfMemoryException,  # noqa:  F401 `cluster_tools.schedulers.cluster_executor.ClusterExecutor` imported but unused;
+    RemoteResourceLimitException,  # noqa:  F401 `cluster_tools.schedulers.cluster_executor.ClusterExecutor` imported but unused;
+    RemoteTimeLimitException,  # noqa:  F401 `cluster_tools.schedulers.cluster_executor.ClusterExecutor` imported but unused;
 )
 from cluster_tools.schedulers.kube import KubernetesExecutor
 from cluster_tools.schedulers.pbs import PBSExecutor
@@ -62,48 +54,45 @@ def _test_valid_multiprocessing() -> None:
 
 
 @overload
-def get_executor(environment: Literal["slurm"], **kwargs: Any) -> SlurmExecutor:
-    ...
+def get_executor(environment: Literal["slurm"], **kwargs: Any) -> SlurmExecutor: ...
 
 
 @overload
-def get_executor(environment: Literal["pbs"], **kwargs: Any) -> PBSExecutor:
-    ...
+def get_executor(environment: Literal["pbs"], **kwargs: Any) -> PBSExecutor: ...
 
 
 @overload
 def get_executor(
     environment: Literal["kubernetes"], **kwargs: Any
-) -> KubernetesExecutor:
-    ...
+) -> KubernetesExecutor: ...
+
+
+@overload
+def get_executor(environment: Literal["dask"], **kwargs: Any) -> DaskExecutor: ...
 
 
 @overload
 def get_executor(
     environment: Literal["multiprocessing"], **kwargs: Any
-) -> MultiprocessingExecutor:
-    ...
+) -> MultiprocessingExecutor: ...
 
 
 @overload
 def get_executor(
     environment: Literal["sequential"], **kwargs: Any
-) -> SequentialExecutor:
-    ...
+) -> SequentialExecutor: ...
 
 
 @overload
 def get_executor(
     environment: Literal["debug_sequential"], **kwargs: Any
-) -> DebugSequentialExecutor:
-    ...
+) -> DebugSequentialExecutor: ...
 
 
 @overload
 def get_executor(
     environment: Literal["test_pickling"], **kwargs: Any
-) -> PickleExecutor:
-    ...
+) -> PickleExecutor: ...
 
 
 def get_executor(environment: str, **kwargs: Any) -> "Executor":
@@ -113,6 +102,11 @@ def get_executor(environment: str, **kwargs: Any) -> "Executor":
         return PBSExecutor(**kwargs)
     elif environment == "kubernetes":
         return KubernetesExecutor(**kwargs)
+    elif environment == "dask":
+        if "client" in kwargs:
+            return DaskExecutor(kwargs["client"])
+        else:
+            return DaskExecutor.from_config(**kwargs)
     elif environment == "multiprocessing":
         global did_start_test_multiprocessing
         if not did_start_test_multiprocessing:
@@ -127,6 +121,3 @@ def get_executor(environment: str, **kwargs: Any) -> "Executor":
     elif environment == "test_pickling":
         return PickleExecutor(**kwargs)
     raise Exception("Unknown executor: {}".format(environment))
-
-
-Executor = Union[ClusterExecutor, MultiprocessingExecutor]
