@@ -15,6 +15,7 @@ from typing import (
     Iterable,
     Iterator,
     List,
+    Literal,
     Optional,
     Tuple,
     Type,
@@ -22,8 +23,9 @@ from typing import (
     Union,
     cast,
 )
+from weakref import ReferenceType, ref
 
-from typing_extensions import Literal, ParamSpec
+from typing_extensions import ParamSpec
 
 from cluster_tools._utils import pickling
 from cluster_tools._utils.file_wait_thread import FileWaitThread
@@ -31,10 +33,10 @@ from cluster_tools._utils.reflection import (
     file_path_to_absolute_module,
     get_function_name,
 )
-from cluster_tools._utils.string import random_string, with_preliminary_postfix
+from cluster_tools._utils.string_ import random_string, with_preliminary_postfix
 from cluster_tools._utils.tailf import Tail
 from cluster_tools._utils.warning import enrich_future_with_uncaught_warning
-from cluster_tools.executors.multiprocessing import CFutDict
+from cluster_tools.executors.multiprocessing_ import CFutDict
 
 NOT_YET_SUBMITTED_STATE_TYPE = Literal["NOT_YET_SUBMITTED"]
 NOT_YET_SUBMITTED_STATE: NOT_YET_SUBMITTED_STATE_TYPE = "NOT_YET_SUBMITTED"
@@ -44,14 +46,24 @@ _P = ParamSpec("_P")
 _S = TypeVar("_S")
 
 
+def _handle_kill_through_weakref(
+    executor_ref: "ReferenceType[ClusterExecutor]",
+    existing_sigint_handler: Any,
+    signum: Optional[int],
+    frame: Any,
+) -> None:
+    executor = executor_ref()
+    if executor is None:
+        return
+    executor.handle_kill(existing_sigint_handler, signum, frame)
+
+
 def join_messages(strings: List[str]) -> str:
     return " ".join(x.strip() for x in strings if x.strip())
 
 
 class RemoteException(Exception):
-    def __init__(
-        self, error: str, job_id: str
-    ):  # pylint: disable=super-init-not-called
+    def __init__(self, error: str, job_id: str):
         self.error = error
         self.job_id = job_id
 
@@ -59,11 +71,19 @@ class RemoteException(Exception):
         return self.error.strip() + f" (job_id={self.job_id})"
 
 
-class RemoteOutOfMemoryException(RemoteException):
+class RemoteResourceLimitException(RemoteException):
     def __str__(self) -> str:
         return str(self.job_id) + "\n" + self.error.strip()
 
 
+class RemoteOutOfMemoryException(RemoteResourceLimitException):
+    pass
+
+
+class RemoteTimeLimitException(RemoteResourceLimitException):
+    pass
+
+
 class ClusterExecutor(futures.Executor):
     """Futures executor for executing jobs on a cluster."""
 
@@ -123,16 +143,23 @@ class ClusterExecutor(futures.Executor):
         # shutdown of the main process which sends SIGTERM signals to terminate all
         # child processes.
         existing_sigint_handler = signal.getsignal(signal.SIGINT)
-        signal.signal(signal.SIGINT, partial(self.handle_kill, existing_sigint_handler))
+        signal.signal(
+            signal.SIGINT,
+            partial(_handle_kill_through_weakref, ref(self), existing_sigint_handler),
+        )
 
-        self.meta_data = {}
+        self.metadata = {}
         assert not (
             "logging_config" in kwargs and "logging_setup_fn" in kwargs
         ), "Specify either logging_config OR logging_setup_fn but not both at once"
         if "logging_config" in kwargs:
-            self.meta_data["logging_config"] = kwargs["logging_config"]
+            self.metadata["logging_config"] = kwargs["logging_config"]
         if "logging_setup_fn" in kwargs:
-            self.meta_data["logging_setup_fn"] = kwargs["logging_setup_fn"]
+            self.metadata["logging_setup_fn"] = kwargs["logging_setup_fn"]
+
+    @classmethod
+    def as_completed(cls, futs: List["Future[_T]"]) -> Iterator["Future[_T]"]:
+        return futures.as_completed(futs)
 
     @classmethod
     @abstractmethod
@@ -151,8 +178,7 @@ class ClusterExecutor(futures.Executor):
         self.wait_thread.stop()
 
         if (
-            existing_sigint_handler  # pylint: disable=comparison-with-callable
-            != signal.default_int_handler
+            existing_sigint_handler != signal.default_int_handler
             and callable(existing_sigint_handler)  # Could also be signal.SIG_IGN
         ):
             existing_sigint_handler(signum, frame)
@@ -173,7 +199,8 @@ class ClusterExecutor(futures.Executor):
         pass
 
     def investigate_failed_job(
-        self, job_id_with_index: str  # pylint: disable=unused-argument
+        self,
+        job_id_with_index: str,  # noqa: ARG002 Unused method argument: `job_id_with_index`
     ) -> Optional[Tuple[str, Type[RemoteException]]]:
         """
         When a job fails, this method is called to investigate why. If a tuple is returned,
@@ -303,16 +330,14 @@ class ClusterExecutor(futures.Executor):
             # We don't try to deserialize pickling output, because it won't exist.
             success = False
 
-            opt_reason_and_exception_cls = (  # pylint: disable=assignment-from-none
-                self.investigate_failed_job(jobid)
-            )
+            opt_reason_and_exception_cls = self.investigate_failed_job(jobid)
             reason = None
             if opt_reason_and_exception_cls is not None:
                 reason, wrapping_exception_cls = opt_reason_and_exception_cls
 
             result = join_messages(
                 [
-                    f"Job submission/execution failed.",
+                    "Job submission/execution failed.",
                     reason or "",
                     f"Please look into the log file at {self.format_log_file_path(self.cfut_dir, jobid)}.",
                 ]
@@ -387,7 +412,7 @@ class ClusterExecutor(futures.Executor):
 
         # Start the job.
         serialized_function_info = pickling.dumps(
-            (__fn, args, kwargs, self.meta_data, output_pickle_path)
+            ((__fn, self.metadata), args, kwargs, output_pickle_path)
         )
         with open(self.format_infile_name(self.cfut_dir, workerid), "wb") as f:
             f.write(serialized_function_info)
@@ -426,9 +451,10 @@ class ClusterExecutor(futures.Executor):
     def get_jobid_with_index(cls, jobid: Union[str, int], index: int) -> str:
         return f"{jobid}_{index}"
 
-    def get_function_pickle_path(self, workerid: str) -> str:
+    def get_function_and_metadata_pickle_path(self, workerid: str) -> str:
         return self.format_infile_name(
-            self.cfut_dir, self.get_workerid_with_index(workerid, "function")
+            self.cfut_dir,
+            self.get_workerid_with_index(workerid, "function-and-metadata"),
         )
 
     @staticmethod
@@ -436,13 +462,17 @@ class ClusterExecutor(futures.Executor):
         return os.path.join(cfut_dir, f"cfut.main_path.{workerid}.txt")
 
     def store_main_path_to_meta_file(self, workerid: str) -> None:
-        with open(self.get_main_meta_path(self.cfut_dir, workerid), "w") as file:
+        with open(
+            self.get_main_meta_path(self.cfut_dir, workerid), "w", encoding="utf-8"
+        ) as file:
             file.write(file_path_to_absolute_module(sys.argv[0]))
 
     def map_to_futures(
         self,
         fn: Callable[[_S], _T],
-        args: Iterable[_S],  # TODO change: allow more than one arg per call
+        args: Iterable[
+            _S
+        ],  # TODO change: allow more than one arg per call # noqa FIX002 Line contains TODO
         output_pickle_path_getter: Optional[Callable[[_S], os.PathLike]] = None,
     ) -> List["Future[_T]"]:
         self.ensure_not_shutdown()
@@ -455,10 +485,12 @@ class ClusterExecutor(futures.Executor):
         futs_with_output_paths = []
         workerid = random_string()
 
-        pickled_function_path = self.get_function_pickle_path(workerid)
-        self.files_to_clean_up.append(pickled_function_path)
-        with open(pickled_function_path, "wb") as file:
-            pickling.dump(fn, file)
+        pickled_function_and_metadata_path = self.get_function_and_metadata_pickle_path(
+            workerid
+        )
+        self.files_to_clean_up.append(pickled_function_and_metadata_path)
+        with open(pickled_function_and_metadata_path, "wb") as file:
+            pickling.dump((fn, self.metadata), file)
         self.store_main_path_to_meta_file(workerid)
 
         for index, arg in enumerate(args):
@@ -482,7 +514,12 @@ class ClusterExecutor(futures.Executor):
                 os.unlink(preliminary_output_pickle_path)
 
             serialized_function_info = pickling.dumps(
-                (pickled_function_path, [arg], {}, self.meta_data, output_pickle_path)
+                (
+                    pickled_function_and_metadata_path,
+                    [arg],
+                    {},
+                    output_pickle_path,
+                )
             )
             infile_name = self.format_infile_name(self.cfut_dir, workerid_with_index)
 
@@ -533,7 +570,6 @@ class ClusterExecutor(futures.Executor):
     ) -> None:
         jobid = jobid_future.result()
         if self.debug:
-
             logging.debug(
                 "Submitted array job {} with JobId {} and {} subjobs.".format(
                     batch_description, jobid, len(futs_with_output_paths)
@@ -582,14 +618,13 @@ class ClusterExecutor(futures.Executor):
         for file_to_clean_up in self.files_to_clean_up:
             try:
                 os.unlink(file_to_clean_up)
-            except OSError:
+            except OSError:  # noqa: PERF203 `try`-`except` within a loop incurs performance overhead
                 pass
         self.files_to_clean_up = []
 
-    # TODO: args should be *iterables, this would be a breaking change!
     def map(  # type: ignore[override]
         self,
-        fn: Callable[_P, _T],
+        fn: Callable[[_S], _T],
         args: Iterable[Any],
         timeout: Optional[float] = None,
         chunksize: Optional[int] = None,
@@ -634,9 +669,12 @@ class ClusterExecutor(futures.Executor):
         """
 
         log_path = self.format_log_file_path(self.cfut_dir, fut.cluster_jobid)  # type: ignore[attr-defined]
+
         # Don't use a logger instance here, since the child process
         # probably already used a logger.
-        log_callback = lambda s: sys.stdout.write(f"(jid={fut.cluster_jobid}) {s}")  # type: ignore[attr-defined]
+        def log_callback(s: str) -> None:
+            sys.stdout.write(f"(jid={fut.cluster_jobid}) {s}")  # type: ignore[attr-defined]
+
         tailer = Tail(log_path, log_callback)
         fut.add_done_callback(lambda _: tailer.cancel())
 
