@@ -1,15 +1,13 @@
-"""Abstracts access to a PBS cluster via its command-line tools.
-"""
+"""Abstracts access to a PBS cluster via its command-line tools."""
+
 import logging
 import os
 import re
 from concurrent.futures import Future
-from typing import Any, Dict, List, Optional, Tuple, Union
-
-from typing_extensions import Literal
+from typing import Any, Dict, List, Literal, Optional, Tuple, Union
 
 from cluster_tools._utils.call import call, chcall
-from cluster_tools._utils.string import random_string
+from cluster_tools._utils.string_ import random_string
 from cluster_tools.schedulers.cluster_executor import ClusterExecutor
 
 # qstat vs. checkjob
@@ -58,7 +56,7 @@ class PBSExecutor(ClusterExecutor):
     def get_job_id_string(cls) -> str:
         return cls.get_current_job_id()
 
-    def inner_handle_kill(self, *args: Any, **kwargs: Any) -> None:
+    def inner_handle_kill(self, *args: Any, **kwargs: Any) -> None:  # noqa: ARG002 Unused method argument: `args`, kwargs
         scheduled_job_ids: List[Union[int, str]] = list(self.jobs.keys())
 
         if len(scheduled_job_ids):
@@ -91,7 +89,7 @@ class PBSExecutor(ClusterExecutor):
         filename = self.get_temp_file_path(
             self.cfut_dir, "_temp_pbs_{}.sh".format(random_string())
         )
-        with open(filename, "w") as f:
+        with open(filename, "w", encoding="utf-8") as f:
             f.write(job)
         jobid_desc, _ = chcall("qsub -V {}".format(filename))
         match = re.search("^[0-9]+", jobid_desc)
@@ -177,7 +175,6 @@ class PBSExecutor(ClusterExecutor):
             )
             return "ignore"
         else:
-
             job_state_search = re.search("job_state = ([a-zA-Z_]*)", stdout)
             if job_state_search:
                 job_state = job_state_search.group(1)
